import java.io.*;
import java.util.List;
import java.util.ArrayList;
import java.math.BigInteger;

public class aerodynamics_petr_slow {
    private static final int MAX_N = 100;
    private static final int MAX_Z = 100;
    private static final int MAX_COORD = 1000;

    private static class Rational implements Comparable<Rational> {
        public final BigInteger p;
        public final BigInteger q;

        public Rational(BigInteger p, BigInteger q) {
            this.p = p;
            this.q = q;
            if (q.signum() != 1)
                throw new RuntimeException();
            BigInteger z = p.gcd(q).abs();
            p = p.divide(z);
            q = q.divide(z);
        }

        public Rational(int x) {
            this(BigInteger.valueOf(x), BigInteger.ONE);
        }

        public Rational(int p, int q) {
            this(BigInteger.valueOf(p), BigInteger.valueOf(q));
        }

        public Rational subtract(Rational o) {
            return new Rational(p.multiply(o.q).subtract(q.multiply(o.p)), q.multiply(o.q));
        }

        public Rational multiply(Rational o) {
            return new Rational(p.multiply(o.p), q.multiply(o.q));
        }

        public Rational add(Rational o) {
            return new Rational(p.multiply(o.q).add(q.multiply(o.p)), q.multiply(o.q));
        }

        public int signum() {
            return p.signum();
        }

        public Rational abs() {
            return new Rational(p.abs(), q);
        }

        public double doubleValue() {
            return p.doubleValue() / q.doubleValue();
        }

        public int compareTo(Rational o) {
            return p.multiply(o.q).compareTo(q.multiply(o.p));
        }

        @Override
        public String toString() {
            return p + "/" + q;
        }
    }

    private static class Point {
        public final Rational x;
        public final Rational y;

        Point(Rational x, Rational y) {
            this.x = x;
            this.y = y;
        }
    }

    public static void main(String[] args) throws IOException {
        BufferedReader reader = new BufferedReader(new FileReader("aerodynamics.in"));
        String[] parts = reader.readLine().split(" ", -1);
        if (parts.length != 3)
            throw new RuntimeException();
        int n = Integer.parseInt(parts[0]);
        if (n < 4 || n > MAX_N)
            throw new RuntimeException();
        int zMin = Integer.parseInt(parts[1]);
        if (zMin < 0 || zMin > MAX_Z)
            throw new RuntimeException();
        int zMax = Integer.parseInt(parts[2]);
        if (zMax < zMin || zMax > MAX_Z)
            throw new RuntimeException();
        int[] x = new int[n];
        int[] y = new int[n];
        int[] z = new int[n];
        for (int i = 0; i < n; ++i) {
            parts = reader.readLine().split(" ", -1);
            if (parts.length != 3)
                throw new RuntimeException();
            x[i] = Integer.parseInt(parts[0]);
            y[i] = Integer.parseInt(parts[1]);
            z[i] = Integer.parseInt(parts[2]);
            if (x[i] < -MAX_COORD || x[i] > MAX_COORD)
                throw new RuntimeException();
            if (y[i] < -MAX_COORD || y[i] > MAX_COORD)
                throw new RuntimeException();
            if (z[i] < -MAX_COORD || z[i] > MAX_COORD)
                throw new RuntimeException();
        }
        if (reader.read() != -1)
            throw new RuntimeException();

        PrintWriter writer = new PrintWriter("aerodynamics.out");
        for (int cz = zMin; cz <= zMax; ++cz) {
            List<Point> cList = new ArrayList<Point>();
            for (int i = 0; i < n; ++i) {
                if (z[i] == cz)
                    cList.add(new Point(new Rational(x[i]), new Rational(y[i])));
                if (z[i] < cz)
                    for (int j = 0; j < n; ++j)
                        if (z[j] > cz) {
                            cList.add(new Point(
                                    new Rational(x[i] * (z[j] - z[i]) + (x[j] - x[i]) * (cz - z[i]), (z[j] - z[i])),
                                    new Rational(y[i] * (z[j] - z[i]) + (y[j] - y[i]) * (cz - z[i]), (z[j] - z[i]))));
                        }
            }
            /*System.out.println(cz);
            for (int i = 0; i < cList.size(); ++i) {
                System.out.print("(" + cList.get(i).x + "," + cList.get(i).y + ") ");
            }
            System.out.println("");*/
            writer.println(hullArea(cList));
        }
        writer.close();
    }

    private static double hullArea(List<Point> points) {
        Point start = null;
        for (Point point : points) {
            if (start == null)
                start = point;
            else {
                int cx = point.x.compareTo(start.x);
                if (cx < 0)
                    start = point;
                else if (cx == 0) {
                    if (point.y.compareTo(start.y) < 0)
                        start = point;
                }
            }
        }
        double res = 0;
        Point cur = start;
        do {
            Point next = null;
            Rational ndx = null;
            Rational ndy = null;
            Rational nlen = null;
            for (Point point : points)
                if (point.x.compareTo(cur.x) != 0 || point.y.compareTo(cur.y) != 0) {
                    Rational dx = point.x.subtract(cur.x);
                    Rational dy = point.y.subtract(cur.y);
                    Rational len = dx.abs().add(dy.abs());
                    if (next != null) {
                        int s = dx.multiply(ndy).subtract(dy.multiply(ndx)).signum();
                        if (s < 0)
                            continue;
                        if (s == 0) {
                            s = len.compareTo(nlen);
                            if (s <= 0)
                                continue;
                        }
                    }
                    ndx = dx;
                    ndy = dy;
                    next = point;
                    nlen = len;
                }
            if (next == null)
                return 0.0;
            res += next.x.add(cur.x).multiply(next.y.subtract(cur.y)).doubleValue();
            cur = next;
        } while (cur.x.compareTo(start.x) != 0 || cur.y.compareTo(start.y) != 0);
        return Math.abs(res) / 2;
    }
}
