#include <cstdio>
#include <cstring>
#include <cstdlib>
#include <cmath>
#include <cassert>
#include <iostream>
#include <sstream>
#include <algorithm>
#include <vector>
#include <set>
#include <map>
#include <queue>
#include <cassert>

using namespace std;

#define sz(v) ((int) (v).size())
#define all(v) (v).begin(), (v).end()
#define mp make_pair
#define pb push_back
#define forn(i,n) for (int i=0; i<n; i++)

typedef long long ll;
typedef long long int64;
typedef pair<int,int> ii;
typedef vector<int> vi;
typedef vector<string> vs;

template<typename T> T abs(T x) { return x>0 ? x : -x; }
template<typename T> T sqr(T x) { return x*x;          }

const double eps = 1e-8;

double det(double x1, double x2, double y1, double y2) {
	return x1 * y2 - x2 * y1;
}

struct P3 {
	double x, y, z;
	P3(): x(0), y(0), z(0) {}
	P3(double x, double y, double z): x(x), y(y), z(z) {}
	P3 operator -(const P3& a) {
		return P3(x - a.x, y - a.y, z - a.z);
	}
	P3 operator +(const P3& a) {
		return P3(x + a.x, y + a.y, z + a.z);
	}
	P3 operator /(double k) {
		return P3(x / k, y / k, z / k);
	}
	double len() {
		return sqrt(sqr(x) + sqr(y) + sqr(z));
	}
	P3 operator *(const P3& a) {
		return P3(det(y, z, a.y, a.z), det(z, x, a.z, a.x), det(x, y, a.x, a.y));
	}
	double operator ^(const P3& a) {
		return x * a.x + y * a.y + z * a.z;
	}
	double vol(const P3& a, const P3& b) {
		return ((*this) * a) ^ b;
	}
	P3 norm() {
		double l = len();
		assert(abs(l) > eps);
		return P3(x / l, y / l, z / l);
	}
	void load() {
		cin >> x >> y >> z;
	}
	double distTo(P3 A, P3 B, P3 C)
	{
		P3 O = *this;
		P3 v = A - O;
		P3 u = B - O;
		P3 w = C - O;
		double V = v * u ^ w;
		double S = ((A - B) * (A - C)).len();
		double dist = fabs(V / S);
		return dist;
	}
};

struct P2 {
	double x, y;
	P2() {}
	P2(double x, double y): x(x), y(y) {}
	P2 operator -(const P2& a) {
		return P2(x - a.x, y - a.y);
	}
	double operator *(const P2& a) {
		return x * a.y - y * a.x;
	}
	double operator ^(const P2& a) {
		return det(x, y, a.x, a.y);
	}
	double len() {
		return sqrt(sqr(x) + sqr(y));
	}
};

class SetOfTriangles {
public:
	void AddTriangle(const P3& a, const P3& b, const P3& c) {
		triangle_coord_vectors_.insert(TriangleToIntVector(a, b, c));
	}
	void AddAllTriangles(const vector<P3>& plane) {
		for (int i = 0; i < sz(plane); ++i)
			for (int j = 0; j < sz(plane); ++j)
				for (int k = 0; k < sz(plane); ++k) {
					AddTriangle(plane[i], plane[j], plane[k]);
				}
	}
	bool ContainsTriangle(const P3& a, const P3&b, const P3& c) {
		return triangle_coord_vectors_.find(TriangleToIntVector(a, b, c)) != triangle_coord_vectors_.end();
	}
private:
	set<vector<int> > triangle_coord_vectors_;
	vi TriangleToIntVector(const P3& a, const P3& b, const P3& c) {
		vi v;
		v.pb(a.x), v.pb(a.y), v.pb(a.z), v.pb(b.x), v.pb(b.y), v.pb(b.z), v.pb(c.x), v.pb(c.y), v.pb(c.z);
		return v;
	}
};

bool OnOneLine(P3 a, P3 b, P3 c) {
	b = b - a;
	c = c - a;
	return (b * c).len() < eps;
}

vector<P2> Get2DVector(vector<P3>& plane) {
	assert(sz(plane) >= 3);
	assert(!OnOneLine(plane[0], plane[1], plane[2]));
	P3 v1 = (plane[1] - plane[0]).norm();
	P3 tmp = (plane[2] - plane[0]).norm();
	P3 ort = (v1 * tmp).norm();
	P3 v2 = v1 * ort;
	vector<P2> res(sz(plane));	
	for (int i = 0; i < sz(plane); ++i) {
		res[i] = P2((plane[i] - plane[0]) ^ v1, (plane[i] - plane[0]) ^ v2);
	}
	return res;
}

double GetArea(vector<P2>& polygon) {
	double res = 0;
	for (int i = 0; i < sz(polygon); ++i) {
		res += polygon[i] * polygon[(i + 1) % sz(polygon)];
	}
	res = abs(res) / 2;
	return res;
}

P2 C;
bool cmp(P2 a, P2 b) {
	a = a - C;
	b = b - C;
	double vct = a * b;
	if (abs(vct) > eps) {
		return vct > 0;
	}
	return a.len() < b.len() - eps;
}
vector<P2> GetConvexHull(vector<P2>& p) {
	assert(sz(p) >= 3);
	for (int i = 0; i < sz(p); ++i) {
		if (p[i].x < p[0].x - eps || abs(p[i].x - p[0].x) < eps && p[i].y < p[0].y - eps) {
			swap(p[0], p[i]);
		}
	}
	C = p[0];
	sort(p.begin() + 1, p.end(), cmp);
	vector<P2> res;
	for (int i = 0; i < sz(p); ++i) {
		while (sz(res) >= 2) {
			P2 p1 = res[sz(res) - 2];
			P2 p2 = res[sz(res) - 1];
			if ((p[i] - p1) * (p2 - p1) > -eps) {
				res.pop_back();
			} else {
				break;
			}
		}
		res.pb(p[i]);
	}
	return res;
}

int n;
vector<P3> p3;
SetOfTriangles set_of_triangles;

bool IsFacetPlane(P3 a, P3 b, P3 c, vector<P3>& plane) {
	if (OnOneLine(a, b, c)) {
		return false;
	}
	bool was_plus = false;
	bool was_minus = false;
	for (int i = 0; i < p3.size(); ++i) {
		double vol = (b - a).vol(c - a, p3[i] - a);
		if (vol > eps) {
			was_plus = true;
		} else if (vol < -eps) {
			was_minus = true;
		} else {
			plane.pb(p3[i]);
		}
	}
	return !was_plus || ! was_minus;
}


double getV(P3 A, P3 B, P3 C, P3 O)
{
	return fabs( (A - O) * (B - O) ^ (C - O) );
}


int main() {
	freopen("input.txt", "r", stdin);
	freopen("output.txt", "w", stdout);
	cin >> n;
	p3.resize(n);
	P3 center = P3();
	for (int i = 0; i < n; ++i) {
		p3[i].load();
		center = center + p3[i];
	}
	center = center / n;
	double res = 0;
	int planes_count = 0;
	for (int i = 0; i < n; ++i)
		for (int j = i + 1; j < n; ++j)
			for (int k = j + 1; k < n; ++k) {
				vector<P3> plane;
				if (!IsFacetPlane(p3[i], p3[j], p3[k], plane)) {
					continue;
				}
				if (set_of_triangles.ContainsTriangle(p3[i], p3[j], p3[k])) {
					continue;
				}
				set_of_triangles.AddAllTriangles(plane);
				planes_count++;
				double dist = center.distTo(p3[i], p3[j], p3[k]);
				vector<P2> p2 = Get2DVector(plane);
				vector<P2> ch = GetConvexHull(p2);
				res += GetArea(ch) * dist;
			}
	printf("%lld\n", (long long) (res * 2 + 0.5));
	return 0;
}
