#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <cassert>

#include <iostream>
#include <iomanip>
#include <string>
#include <vector>
#include <set>
#include <map>
#include <deque>
#include <queue>
#include <stack>
#include <algorithm>
#include <cassert>

#define forn(i, n) for(int i = 0; i < int(n); i++)
#define mp make_pair
#define pb push_back
#define foreach(it, v) for(__typeof(v.begin()) it = v.begin(); it != v.end(); ++it)

using namespace std;

typedef long long ll;

struct Point {
	int x, y, id, sortOrder;

	bool operator < (const Point & other) const {
		if(x != other.x)
			return x < other.x;
		return y < other.y;
	}
	
	bool operator == (const Point & other) const {
		return (x == other.x && y == other.y);
	}
};

struct ConvexCompare {
	const Point & basePoint;
	
	ConvexCompare(const Point & basePoint) : basePoint(basePoint) {}
	
	inline bool operator () (const Point & a, const Point & b) const {
		int ax = a.x - basePoint.x, ay = a.y - basePoint.y;
		int bx = b.x - basePoint.x, by = b.y - basePoint.y;
		int mult = ax * by - ay * bx;
		if(mult != 0) return mult < 0;
		int da = ax * ax + ay * ay;
		int db = bx * bx + by * by;
		return da < db;
	}
};

const int kMaxValue = 20000;

int counts[kMaxValue];
Point countSortBuffer[kMaxValue];

int convexHullStackSize;
Point convexHullStack[kMaxValue];

int pointsSizes[2];
Point points[2][kMaxValue];

int results[kMaxValue];

template < typename T >
bool isSorted(int allPointsId, T compare) {
	int n = pointsSizes[allPointsId];
	for(int i = 1; i < n; i++) {
		if(!compare(points[allPointsId][i - 1], points[allPointsId][i])) {
			return false;
		}
	}
	return true;
}



bool isBad(const Point & newPoint) {
	assert(convexHullStackSize >= 2);
	const Point & previousPoint = convexHullStack[convexHullStackSize - 2];
	const Point & currentPoint = convexHullStack[convexHullStackSize - 1];
	return ConvexCompare(previousPoint)(newPoint, currentPoint);
}

void countSort(int newPointsId, int maxValue) {
	fill(counts, counts + maxValue, 0);
	int n = pointsSizes[newPointsId];
	for(int i = 0; i < n; i++) {
		const Point & point = points[newPointsId][i];
		counts[point.sortOrder]++;
	}
	for(int i = 1; i < maxValue; i++) {
		counts[i] += counts[i - 1];
	}
	for(int i = n - 1; i >= 0; i--) {
		const Point & point = points[newPointsId][i];
		countSortBuffer[--counts[point.sortOrder]] = point;
	}
	copy(countSortBuffer, countSortBuffer + n, points[newPointsId]);
}

int remove(int allPointsId, const Point & basePoint) {
	int newSize = 0, isSame = 0;
	for(int i = 0; i < pointsSizes[allPointsId]; i++) {
		Point p = points[allPointsId][i];
		if(p == basePoint) {
			++isSame;
		} else {
			points[allPointsId][newSize++] = p;
		}
	}
	pointsSizes[allPointsId] = newSize;
	return isSame;
}

void buildConvexHull(int allPointsId, int newPointsId) {
	//cout << "RUN" << endl;
	Point basePoint = *min_element(points[allPointsId], points[allPointsId] + pointsSizes[allPointsId]);
	int isSame = remove(allPointsId, basePoint);
	auto compare = ConvexCompare(basePoint);
	if(!isSorted(allPointsId, compare)) {
		sort(points[allPointsId], points[allPointsId] + pointsSizes[allPointsId], compare);
		//cout << "SORT" << endl;
	}
	convexHullStackSize = 0;
	convexHullStack[convexHullStackSize++] = basePoint;
	pointsSizes[newPointsId] = 0;
	for(int i = 0; i < pointsSizes[allPointsId]; i++) {
		Point & currentPoint = points[allPointsId][i];
		
		currentPoint.sortOrder = i;
		//cout << "Point: " << currentPoint.x << " " << currentPoint.y << endl;

		while(convexHullStackSize >= 2 && isBad(currentPoint)) {
			const Point & pointToRemove = convexHullStack[convexHullStackSize - 1];
			points[newPointsId][pointsSizes[newPointsId]++] = pointToRemove;
			--convexHullStackSize;
		}
		
		convexHullStack[convexHullStackSize++] = currentPoint;
	
	}
	countSort(newPointsId, pointsSizes[allPointsId]);
	assert(isSame == 1);
}

int main() {
	ios_base::sync_with_stdio(false);
	double startTime = clock();
	int pointsCount;
	cin >> pointsCount;
	pointsSizes[0] = pointsCount;
	for(int i = 0; i < pointsCount; i++) {
		cin >> points[0][i].x >> points[0][i].y;
		points[0][i].id = i;
	}
	sort(points[0], points[0] + pointsCount);
	int currentId = 0, freeId = 1;
	for(int it = 1; pointsSizes[currentId] != 0; it++) {
		buildConvexHull(currentId, freeId);
		swap(currentId, freeId);
		//cout << convexHullStackSize << endl;
		for(int i = 0; i < convexHullStackSize; i++ ) {
			const Point & p = convexHullStack[i];
			results[p.id] = it;
		}
	}
	for(int i = 0; i < pointsCount; i++) {
		cout << results[i] << "\n";
	}
	double resultTime = (clock() - startTime) / CLOCKS_PER_SEC;
	//cerr << resultTime << endl;
	return 0;
}