#include <iostream>
#include <iomanip>
#include <cmath>
using namespace std;

const double eps = 1e-9;

template<typename T>
inline T sqr(T x)
{
    return x * x;
}

struct Point
{
    double x, y, z;

    Point() :
        x(0.0),
        y(0.0),
        z(0.0)
    {
    }

    Point(double x, double y, double z) :
        x(x),
        y(y),
        z(z)
    {
    }
};

double Length(const Point& point)
{
    return sqrt(sqr(point.x) + sqr(point.y) + sqr(point.z));
}

istream& operator >> (istream& input_stream, Point& point)
{
    input_stream >> point.x >> point.y >> point.z;
    return input_stream;
}

Point operator - (const Point& lhs, const Point& rhs)
{
    return Point(lhs.x - rhs.x, lhs.y - rhs.y, lhs.z - rhs.z);
}

Point VMult(const Point& lhs, const Point& rhs)
{
    return Point(lhs.y * rhs.z - lhs.z * rhs.y,
                  -(lhs.x * rhs.z - rhs.x * lhs.z),
                 lhs.x * rhs.y - rhs.x * lhs.y);
}

double SMult(const Point& lhs, const Point& rhs)
{
    return lhs.x * rhs.x + lhs.y * rhs.y + lhs.z * rhs.z;
}

double GetDoubledTriangleSquare(const Point& A, const Point& B, const Point& C)
{
    return Length(VMult(C - A, C - B));
}


double GetDistToPlane(const Point& A, const Point& B, const Point& C,
                    const Point& P)
{
    double V = SMult(VMult(P - A, P - B), P - C);

    double S = GetDoubledTriangleSquare(A, B, C);

    return fabs(V / S);
}

int main()
{
    ios_base::sync_with_stdio(false);

    double mp;
    double h, H;

    cin >> mp >> h >> H;

    Point P[5];

    for (int i = 0; i <= 4; ++i) {
        P[i].z = 0;
    }
    P[4].z = H;
    P[0].z = h;

    for (int i = 0; i <= 4; ++i) {
        cin >> P[i].x >> P[i].y;
    }

    double min_dist = GetDistToPlane(P[1], P[2], P[3], P[0]);

    min_dist = min(min_dist, GetDistToPlane(P[1], P[2], P[4], P[0]));
    min_dist = min(min_dist, GetDistToPlane(P[1], P[3], P[4], P[0]));
    min_dist = min(min_dist, GetDistToPlane(P[2], P[3], P[4], P[0]));

    if (min_dist + h - eps < mp) {
        cout << "YES" << endl;
    } else {
        cout << "NO" << endl;
    }

    return 0;
}
