#include <iostream>
#include <algorithm>
#include <vector>

using namespace std;

struct point_t {
    int x, y, number, current_number;

    point_t(int x = 0, int y = 0, int number = 0) :
        x(x),
        y(y),
        number(0),
        current_number(0) {
    }
};

struct cmp_t {
    bool operator () (point_t const &a, point_t const &b) const {
        return a.x < b.x || (a.x == b.x && a.y < b.y);
    }
};

int mult(point_t const &a, point_t const &b, point_t const &c) {
    return (b.x - a.x) * (c.y - b.y) - (c.x - b.x) * (b.y - a.y);
}

int n;
vector<char> used;
vector<int> answer;
vector<point_t> points, points1, up, down;

void convex_hull() {
    up.clear();
    down.clear();
    up.push_back(points[0]);
    down.push_back(points[0]);
    for (int i = 1; i < (int) points.size(); ++i) {
        if (mult(points[0], points[i], points.back()) <= 0) {
            while (up.size() >= 2 && mult(up[up.size() - 2], up[up.size() - 1], points[i]) > 0) {
                up.pop_back();
            }
            up.push_back(points[i]);
        }
        if (mult(points[0], points[i], points.back()) >= 0) {
            while (down.size() >= 2 && mult(down[down.size() - 2], down[down.size() - 1], points[i]) < 0) {
                down.pop_back();
            }
            down.push_back(points[i]);
        }
    }
}

#define DEBUG_

int main() {
#ifdef DEBUG
    freopen("input.txt", "r", stdin);
#endif
    ios_base::sync_with_stdio(false);
    cin >> n;
    points.resize(n);
    up.reserve(n);
    down.reserve(n);
    used.resize(n, false);
    answer.resize(n, 0);
    points1.reserve(n);
    for (int i = 0; i < n; ++i) {
        cin >> points[i].x >> points[i].y;
        points[i].number = i;
        points[i].current_number = i;
    }
    int current_answer = 0;
    sort(points.begin(), points.end(), cmp_t());
    while (!points.empty()) {
        ++current_answer;
#ifdef DEBUG
        cout << current_answer << endl;
#endif
        convex_hull();
        for (int i = 0; i < (int) up.size(); ++i) {
#ifdef DEBUG
            cout << ": " << up[i].x << ' ' << up[i].y << endl;
#endif
            used[up[i].current_number] = true;
            answer[up[i].number] = current_answer;
        }
        for (int i = 0; i < (int) down.size(); ++i) {
#ifdef DEBUG
            cout << "- " << down[i].x << ' ' << down[i].y << endl;
#endif
            used[down[i].current_number] = true;
            answer[down[i].number] = current_answer;
        }
        for (int i = 0; i < (int) points.size(); ++i) {
            if (!used[points[i].current_number]) {
                points1.push_back(points[i]);
            }
        }
        points.clear();
        swap(points, points1);
        for (int i = 0; i < (int) points.size(); ++i) {
            points[i].current_number = i;
            used[i] = false;
        }
    }
    for (int i = 0; i < (int) answer.size(); ++i) {
        cout << answer[i] << '\n';
    }
    return 0;
}
