#include <iostream>
#include <algorithm>
#include <cstring>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <iomanip>

using namespace std;

const int MAX_N = 10005;
long double eps = 1e-13;

struct Point3D {
    long double x, y, z;

    Point3D(long double X = 0, long double Y = 0, long double Z = 0): x(X), y(Y), z(Z) {
    }
};

struct Point2D {
    long double x, y;

    Point2D(long double X = 0, long double Y = 0): x(X), y(Y) {
    }
};

int n, z1, z2;
Point3D p[MAX_N];

bool equal(long double a, long double b) {
    return fabs(a - b) < eps;
}

bool greatr(long double a, long double b) {
    if (a > b && !equal(a, b)) {
        return true;
    }

    return false;
}

bool greaterEqual(long double a, long double b) {
    if (a > b || equal(a, b)) {
        return true;
    }

    return false;
}

Point2D intersect(Point3D p1, Point3D p2, int z) {
    if (greatr(p1.z, p2.z)) {
        swap(p1, p2);
    }

    return Point2D(p1.x + (p2.x - p1.x) * (z - p1.z) / (p2.z - p1.z),
            p1.y + (p2.y - p1.y) * (z - p1.z) / (p2.z - p1.z));
}

long double ccw(Point2D a, Point2D b, Point2D c) {
    long double x1 = a.x - b.x, x2 = c.x - b.x;
    long double y1 = a.y - b.y, y2 = c.y - b.y;

    return x1 * y2 - x2 * y1;
}

long double area(Point2D p[], int n) {
    int mn = 0;
    pair< pair< long double, pair< long double, long double > >, int > s[MAX_N];

    for (int i = 0; i < n; i++) {
        if (greatr(p[mn].y, p[i].y)) {
            mn = i;
        } else if (equal(p[i].y, p[mn].y)  && greatr(p[mn].x, p[i].x)) {
            mn = i;
        }
    }

    for (int i = 0; i < n; i++) {
        s[i] = make_pair(make_pair(atan2(p[i].y - p[mn].y, p[i].x - p[mn].x), make_pair(p[i].y, p[i].x)), i);
    }

    sort(s, s + n);

    int sz = 0, hull[MAX_N];

    for (int i = 0; i < n; i++) {
        int u = s[i].second;

        while (sz > 1 && greaterEqual(ccw(p[hull[sz - 2]], p[hull[sz - 1]], p[u]), 0.0)) {
            sz--;
        }

        hull[sz++] = u;
    }
    
    while (sz >= 3 && greaterEqual(ccw(p[hull[sz - 2]], p[hull[sz - 1]], p[hull[0]]), 0.0)) {
        sz--;
    }

    long double ans = 0.0;

    for (int j = 1; j < sz - 1; j++) {
        ans += fabs(ccw(p[hull[0]], p[hull[j]], p[hull[j + 1]])) / 2.0;
    }

    return ans;
}

int main() {
    cin >> n >> z1 >> z2;

    for (int i = 0; i < n; i++) {
        cin >> p[i].x >> p[i].y >> p[i].z;
    }

    for (int z = z1; z <= z2; z++) {
        int sz = 0;
        Point2D cur_p[MAX_N];

        for (int i = 0; i < n; i++) {
            for (int j = i + 1; j < n; j++) {
                if ((greaterEqual(z, p[i].z) && greaterEqual(p[j].z, z)) || (greaterEqual(p[i].z, z) && greaterEqual(z, p[j].z))) {
                    if (!equal(p[i].z, z) || !equal(p[j].z, z)) {
                        cur_p[sz++] = intersect(p[i], p[j], z);
                    }
                }

            }

            if (equal(p[i].z, z)) {
                cur_p[sz++] = Point2D(p[i].x, p[i].y);
            }
        }

        cout << setprecision(12) << fixed << area(cur_p, sz) << endl;
    }

    return 0;
}
