#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <iostream>
#include <string>
#include <vector>
#include <memory.h>
#include <algorithm>
#include <map>
#include <set>
#include <queue>

using namespace std;

#define fore(i, b, e) for (int i = (int)b; i <= (int)e; i++)
#define forn(i, n) for (int i = 0; i < (int)n; i++)
#define fi first
#define se second
#define pii pair <int, int>
#define mp make_pair
#define i64 long long
#define li long long
#define vi vector <int>
#define pb push_back

struct Point3d
{
	double x, y, z;
	Point3d(){}
	Point3d(Point3d p1, Point3d p2) : x(p2.x - p1.x), y(p2.y - p1.y), z(p2.z - p1.z) {}
};

struct Point2d
{
	double x, y;
	Point2d(double _x, double _y) : x(_x), y(_y) {}
	Point2d(Point2d p1, Point2d p2) : x(p2.x - p1.x), y(p2.y - p1.y) {}
	double length()
	{
		return sqrt(x * x + y * y);
	}
	double operator % (Point2d an)
	{
		return x * an.y - y * an.x;
	}
};

const int maxn = 100005;

const double eps = 1.0e-6;

vector <Point2d> list;

bool operator < (Point2d first, Point2d second)
{
	Point2d v1(list[0], first);
	Point2d v2(list[0], second);
	double pr = v1 % v2;
	if (abs(pr) < eps)
		return v1.length() < v2.length();
	return pr > eps;
}

int st[maxn];
int sz;
Point3d p[maxn];

void get_conhull()
{
	st[0] = 0;
	st[1] = 1;
	sz = 2;
	fore(i, 2, list.size() - 1)
	{
		while(sz > 1)
		{
			Point2d prev(list[st[sz - 2]], list[st[sz - 1]]);
			Point2d nnew(list[st[sz - 1]], list[i]);
			double pr = prev % nnew;
			if (pr < -eps || ((abs(pr) < eps) && prev.length() < nnew.length()))
			{
				sz--;
			}
			else break;
		}
		st[sz] = i;
		sz++;
	}
}

inline double get_area()
{
	double ret = 0.0;
	st[sz] = st[0];
	forn(j, sz)
	{
		ret += (list[st[j]] % list[st[j + 1]]);
	}
	return abs(ret) / 2.0;
}

int main()
{
	freopen("aerodynamics.in", "r", stdin);
	freopen("aerodynamics.out", "w", stdout);
	int zmin, zmax;
	int n;
	scanf("%d%d%d", &n, &zmin, &zmax);
	forn(i, n)
		scanf("%lf%lf%lf", &p[i].x, &p[i].y, &p[i].z);
	for (int z0 = zmin; z0 <= zmax; z0++)
	{
		list.clear();
		forn(first, n)
			forn(second, n) if ((first != second) && (z0 - p[first].z >= -eps && p[second].z - z0 >= -eps)
		&& (abs(p[first].z - p[second].z) > eps))
			{
				Point3d v(p[first], p[second]);
				double coeff = (z0 - p[first].z) / (p[second].z - p[first].z);
				list.pb(Point2d(p[first].x + v.x * coeff, p[first].y + v.y * coeff));
			}
		if (list.size() <= 2)
		{
			printf("0.00000\n");
			continue;
		}
		int chosen = 0;
		fore(j, 1, list.size() - 1)
			if (list[j].y < list[chosen].y || (abs(list[j].y - list[chosen].y) < eps && list[j].x < list[chosen].x))
				chosen = j;
		swap(list[0], list[chosen]);
		sort(list.begin()++, list.end());
		get_conhull();
		double area = get_area();
		printf("%.10lf\n", area);
	}
}