#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cassert>
#include <ctime>
#include <cstring>
#include <algorithm>
#include <iostream>
#include <set>
#include <map>
#include <vector>
#include <queue>
#include <deque>
using namespace std;
#define pb push_back
#define mp make_pair
#define sc second
#define fs first
#define int64 long long
#define ldb long double
#define sz(a) ((int) (a).size())
#define eprintf(...) fprintf(stderr, __VA_ARGS__)

const double eps = 1e-8;
const int size = 200;

inline double sqr(double x) {
	return x * x;
}

struct pt {
	double x, y;
	pt(double x, double y): x(x), y(y) {}
	pt() {}
} base;

pt operator - (const pt& a, const pt& b) {
	return pt(a.x - b.x, a.y - b.y);
}

pt operator + (const pt& a, const pt& b) {
	return pt(a.x + b.x, a.y + b.y);
}

double det2(const double& a, const double& b, const double& c, const double& d) {
	return a * d - b * c;
}

int sign(const double& x) {
	return ((x > eps) ? 1 : ((x < -eps) ? -1 : 0));
}

double vmul(const pt& a, const pt& b) {
	return det2(a.x, a.y, b.x, b.y);
}

double dist(const pt& a, const pt& b) {
	return sqrt(sqr(a.x - b.x) + sqr(a.y - b.y));
}


bool cmp_base(const pt& a, const pt& b) {
	int vm = sign(vmul(a - base, b - base));
	if (vm != 0) return (vm > 0);
	return dist(a, base) < dist(b, base);
}

int n, zmin, zmax;

vector<pt> hull(vector<pt> p) {
	if (p.empty())
		return p;
	int n = sz(p);
	for (int i = 0; i < n; ++i)
		if (mp(p[i].x, p[i].y) < mp(p[0].x, p[0].y)) swap(p[i], p[0]);
	base = p[0];
	sort(p.begin() + 1, p.begin() + n, cmp_base);
	vector<pt> q;
	q.pb(p[0]);
	for (int i = 1; i < sz(p); ++i) {
		while ((sz(q) >= 2) && (sign(vmul(p[i] - q[sz(q) - 1], q[sz(q) - 1] - q[sz(q) - 2])) >= 0))
			q.pop_back();
		q.pb(p[i]);
	}

	return q;
}

double getsquare(vector<pt> p) {
	double sum = 0.0;
	for (int i = 0; i < (int) p.size(); i++)
		sum += vmul(p[i], p[(i + 1) % (int) p.size()]);

	return sum;
}

vector <pt> convexes[size];
pt pts[size];
double x[size], y[size], z[size];

int main() {
	//assert(freopen("input.txt", "r", stdin));
	//assert(freopen("output.txt", "w", stdout));

	scanf("%d%d%d", &n, &zmin, &zmax);

	for (int i = 0; i < n; i++) {
		scanf("%lf%lf%lf", &x[i], &y[i], &z[i]);
	}

	for (int i = 0; i < n; i++)
		for (int j = zmin; j <= zmax; j++)
			if (fabs(z[i] - j) < eps)
				convexes[j].pb(pt(x[i], y[i]));

	for (int i = 0; i < n; i++)
		for (int j = 0; j < n; j++) {
			for (int k = zmin; k <= zmax; k++) {
				if (z[i] > k + eps && z[j] < k - eps) {
					double h1 = z[i] - k;
					double h2 = k - z[j];
					convexes[k].pb(pt(x[i] + (x[j] - x[i]) * h1 / (h1 + h2), y[i] + (y[j] - y[i]) * h1 / (h1 + h2)));
				}
			}
		}

	for (int i = zmin; i <= zmax; i++) {
		printf("%.10lf\n", getsquare(hull(convexes[i])) / 2.0);
	}

	return 0;
}