#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cassert>
#include <ctime>
#include <cstring>
#include <algorithm>
#include <iostream>
#include <set>
#include <map>
#include <vector>
#include <queue>
#include <deque>
using namespace std;
#define pb push_back
#define mp make_pair
#define sc second
#define fs first
#define int64 long long
#define ldb long double
#define sz(a) ((int) (a).size())
#define eprintf(...) fprintf(stderr, __VA_ARGS__)
const int N = 100;
const double eps = 1e-10;
const double INF = 1e18;

double sqr(double x) {
	return x * x;
}

struct pt3d {
	double x, y, z;
	pt3d(double x, double y, double z): x(x), y(y), z(z) {}
	pt3d() {}
	void read() {
		scanf("%lf%lf%lf", &x, &y, &z);
	}
	void norm() {
		double d = sqrt(sqr(x) + sqr(y) + sqr(z));
		x /= d, y /= d, z /= d;
	}
} p[N];

struct pt {
	double x, y;
	pt(double x, double y): x(x), y(y) {}
	pt() {}
} base;

pt operator-(const pt& a, const pt& b) {
	return pt(a.x - b.x, a.y - b.y);
}

pt3d operator-(const pt3d& a, const pt3d& b) {
	return pt3d(a.x - b.x, a.y - b.y, a.z - b.z);
}

pt3d operator+(const pt3d& a, const pt3d& b) {
	return pt3d(a.x + b.x, a.y + b.y, a.z + b.z);
}

double det2(const double& a, const double& b, const double& c, const double& d) {
	return a * d - b * c;
}

double det3(const pt3d& a, const pt3d& b, const pt3d& c) {
	return a.x * det2(b.y, b.z, c.y, c.z) - a.y * det2(b.x, b.z, c.x, c.z) + a.z * det2(b.x, b.y, c.x, c.y);
}

pt3d vmul(const pt3d& b, const pt3d& c) {
	return pt3d(det2(b.y, b.z, c.y, c.z), -det2(b.x, b.z, c.x, c.z), det2(b.x, b.y, c.x, c.y));
}

double vmul(const pt& a, const pt& b) {
	return det2(a.x, a.y, b.x, b.y);
}

double dist(const pt& a, const pt& b) {
	return sqrt(sqr(a.x - b.x) + sqr(a.y - b.y));
}

pt3d operator*(const pt3d& a, double d) {
	return pt3d(a.x * d, a.y * d, a.z * d);
}

int sign(const double& x) {
	return ((x > eps) ? 1 : ((x < -eps) ? -1 : 0));
}

double smul(const pt3d& b, const pt3d& c) {
	return b.x * c.x + b.y * c.y + b.z * c.z;
}

bool cmp_base(const pt& a, const pt& b) {
	int vm = sign(vmul(a - base, b - base));
	if (vm != 0) return (vm > 0);
	return dist(a, base) < dist(b, base);
}

vector<pt> hull(vector<pt> p) {
	int n = sz(p);
	for (int i = 0; i < n; ++i)
		if (mp(p[i].x, p[i].y) < mp(p[0].x, p[0].y)) swap(p[i], p[0]);
	base = p[0];
	sort(p.begin() + 1, p.begin() + n, cmp_base);
	vector<pt> q;
	q.pb(p[0]);
	for (int i = 1; i < sz(p); ++i) {
		while ((sz(q) >= 2) && (sign(vmul(p[i] - q[sz(q) - 1], q[sz(q) - 1] - q[sz(q) - 2])) >= 0))
			q.pop_back();
		q.pb(p[i]);
	}
	return q;
}

pt3d get_center(const pt3d& a, const pt3d& b, const pt3d& c, const pt3d& d) {
	return (a + b + c + d) * (1.0 / 4.0);
}

int n;

int main() {
	//assert(freopen("input.txt", "r", stdin));
	//assert(freopen("output.txt", "w", stdout));
	double res = 0;
	for (int it = 0; it < 2; ++it) {
		scanf("%d", &n);
		for (int i = 0; i < n; ++i)
			p[i].read();
		vector< vector<pt3d> > faces;
		for (int i = 0; i < n; ++i)
			for (int j = i + 1; j < n; ++j)
				for (int k = j + 1; k < n; ++k) {
					bool used_neg = false, used_pos = false;
					pt3d v1 = p[j] - p[i], v2 = p[k] - p[i];
					v2 = vmul(vmul(v1, v2), v1);
					v1.norm(), v2.norm();
					vector<pt> pts;
					vector<int> nums;
					for (int l = 0; l < n; ++l) {
						pt3d v = p[l] - p[i];
						int sgn = sign(det3(v, v1, v2));
						if (sgn == -1) used_neg = true;
						else if (sgn == 1) used_pos = true;
						else {
							nums.pb(l);
							pts.pb(pt(smul(v, v1), smul(v, v2)));
						}
					}
					if ((nums[0] != i) || (nums[1] != j) || (nums[2] != k)) continue;
					if (used_pos && used_neg) continue;
					pts = hull(pts);
					vector<pt3d> npts(sz(pts));
					for (int l = 0; l < sz(pts); ++l)
						npts[l] = p[i] + v1 * pts[l].x + v2 * pts[l].y;
					faces.pb(npts);
				}
		pt3d center(0, 0, 0);
		double sum = 0;
		for (int i = 0; i < sz(faces); ++i)
			for (int j = 0; j + 1 < sz(faces[i]); ++j) {
				double volume = fabs(det3(p[0] - faces[i][0], p[0] - faces[i][j], p[0] - faces[i][j + 1]));
				center = center + get_center(p[0], faces[i][0], faces[i][j], faces[i][j + 1]) * volume;
				sum += volume;
			}
		center = center * (1.0 / sum);
		double dist = INF;
		for (int i = 0; i < sz(faces); ++i) {
			pt3d norm = vmul(faces[i][1] - faces[i][0], faces[i][2] - faces[i][0]);
			norm.norm();
			dist = min(dist, fabs(smul(center - faces[i][0], norm)));
		}
		res += dist;
	}
	printf("%.10lf\n", res);
	return 0;
}