#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cassert>
#include <ctime>
#include <cstring>
#include <algorithm>
#include <iostream>
#include <set>
#include <map>
#include <vector>
#include <queue>
#include <deque>
using namespace std;
#define pb push_back
#define mp make_pair
#define sc second
#define fs first
#define int64 long long
#define ldb long double
#define sz(a) ((int) (a).size())
#define eprintf(...) fprintf(stderr, __VA_ARGS__)
const int N = 200;
const double eps = 1e-10;
const double INF = 1e18;

double sqr(double x) {
	return x * x;
}

struct pt3d {
	double x, y, z;
	pt3d(double x, double y, double z): x(x), y(y), z(z) {}
	pt3d() {}
	void read() {
		scanf("%lf%lf%lf", &x, &y, &z);
	}
	void norm() {
		double d = sqrt(sqr(x) + sqr(y) + sqr(z));
		x /= d, y /= d, z /= d;
	}
} p[N];

pt3d operator-(const pt3d& a, const pt3d& b) {
	return pt3d(a.x - b.x, a.y - b.y, a.z - b.z);
}

pt3d operator+(const pt3d& a, const pt3d& b) {
	return pt3d(a.x + b.x, a.y + b.y, a.z + b.z);
}

double det2(const double& a, const double& b, const double& c, const double& d) {
	return a * d - b * c;
}

double det3(const pt3d& a, const pt3d& b, const pt3d& c) {
	return a.x * det2(b.y, b.z, c.y, c.z) - a.y * det2(b.x, b.z, c.x, c.z) + a.z * det2(b.x, b.y, c.x, c.y);
}

pt3d vmul(const pt3d& b, const pt3d& c) {
	return pt3d(det2(b.y, b.z, c.y, c.z), -det2(b.x, b.z, c.x, c.z), det2(b.x, b.y, c.x, c.y));
}

pt3d operator*(const pt3d& a, double d) {
	return pt3d(a.x * d, a.y * d, a.z * d);
}

int sign(const double& x) {
	return ((x > eps) ? 1 : ((x < -eps) ? -1 : 0));
}

double smul(const pt3d& b, const pt3d& c) {
	return b.x * c.x + b.y * c.y + b.z * c.z;
}

const double pi = acos(-1.0);
const int max_it = 200;
int n;
double a, b;

double angle(const pt3d& a, const pt3d& b) {
	return acos(smul(a, b));
}

double geta(double alpha, double beta) {
	return acos(max(-1.0, min(1.0, cos(beta) / cos(alpha))));
}

bool check(double q) {
	vector<pt3d> a;
	for (int i = 0; i < n; ++i)
		for (int j = i + 1; j < n; ++j) {
			double alpha = angle(p[i], p[j]) / 2;
			if ((alpha < eps) || (q < alpha - eps) || (q > pi - alpha + eps) || (alpha > pi - eps)) continue;
			double teta = geta(alpha, q);
			pt3d tmp1 = (p[i] + p[j]) * 0.5;
			tmp1.norm();
			pt3d tmp2 = vmul(p[i], p[j]);
			tmp2.norm();
			a.pb(tmp1 * cos(teta) + tmp2 * sin(teta));
			a.pb(tmp1 * cos(teta) - tmp2 * sin(teta));
		}
	for (int i = 0; i < n; ++i) {
		pt3d tmp(rand(), rand(), rand());
		tmp.norm();
		pt3d ort = vmul(tmp, p[i]);
		ort.norm();
		a.pb(p[i] * cos(q) + ort * sin(q));
	}
	for (int i = 0; i < sz(a); ++i) {
		bool covered = false;
		for (int j = 0; j < n; ++j)
			if (angle(a[i], p[j]) < q - eps) {
				covered = true;
				break;
			}
		if (!covered) return false;
	}
	return true;
}

int main() {
	//assert(freopen("input.txt", "r", stdin));
	//assert(freopen("output.txt", "w", stdout));
	scanf("%d", &n);
	for (int i = 0; i < n; ++i) {
		scanf("%lf%lf", &a, &b);
		a *= pi / 180, b *= pi / 180;
		p[i] = pt3d(cos(a) * cos(b), cos(a) * sin(b), sin(a));
	}
	double l = 0, r = pi;
	for (int it = 0; it < max_it; ++it) {
		double q = (l + r) / 2;
		(check(q) ? r : l) = q;
	}
	printf("%.10lf\n", (l + r) / 2.0);
	return 0;
}