#include <iostream>
#include <algorithm>
#include <string>
#include <vector>
#include <set>
#include <map>
#include <queue>
#include <bitset>
#include <sstream>
#include <cstdio>
#include <cstring>
#include <cmath>
#include <climits>
#include <cctype>
#include <cstdlib>
using namespace std;

#define FOR(i,a,b) for(int i=(a);i<(b);i++)
#define ROF(i,a,b) for(int i=(b)-1;i>=(a);i--)

typedef long long LL;
typedef LL ll;
typedef pair<int, int> pii;

const int kInf = 1000 * 1000 * 1000;
const int INF = kInf;

struct Vector {
  double x, y, z;

  Vector(double x = 0.0, double y = 0.0, double z = 0.0) : x(x), y(y), z(z) {}
};

Vector operator+(const Vector& u, const Vector& v) {
  return Vector(u.x + v.x, u.y + v.y, u.z + v.z);
}

Vector operator-(const Vector& u, const Vector& v) {
  return Vector(u.x - v.x, u.y - v.y, u.z - v.z);
}

double dot(const Vector& u, const Vector& v) {
  return u.x * v.x + u.y * v.y + u.z * v.z;
}
Vector cross(Vector const& a, Vector const& b) {
  return Vector(a.y * b.z - a.z * b.y,
                a.z * b.x - a.x * b.z,
                a.x * b.y - a.y * b.x);
}
Vector operator - (Vector const& a) {
  return Vector(-a.x, -a.y, -a.z);
}

Vector operator*(double k, const Vector& u) {
  return Vector(k * u.x, k * u.y, k * u.z);
}
Vector operator*(const Vector& u, double k) {
  return Vector(k * u.x, k * u.y, k * u.z);
}

typedef Vector Point;

const ll LINF = 1e18;

Vector p[5];
Vector norm[3];

int main() {
  int m, h, H;
  cin >> m >> h >> H;
  FOR(i, 0, 5) cin >> p[i].x >> p[i].y;
  p[4].z = H;
  p[0].z = h;
  double r = 1e+9;
  FOR(i, 0, 3) {
    int a = i + 1;
    int b = (i + 1) % 3 + 1;
    norm[i] = cross(p[4] - p[a], p[4] - p[b]);
    norm[i] = norm[i] * (1 / sqrt(dot(norm[i], norm[i])));
//    cout << norm[i].x << " " << norm[i].y << " " << norm[i].z << endl;
    r = min(r, fabs(dot(p[4], norm[i]) - dot(norm[i], p[0])));
  }
//  cout << r << endl;
  if (r <= (m - h) + 1e-6) {
    printf("YES");
  } else {
    printf("NO");
  }
  return 0;
}
