#include <iostream>
#include <cstdio>
#include <vector>
#include <map>
#include <algorithm>
#include <set>
#include <cstdlib>
#include <cstring>
#include <cmath>

using namespace std;

const long double EPS = 1e-9;

struct Point {
    long double x, y, z;

    Point() {}

    Point(long double x, long double y, long double z): x(x), y(y), z(z) {}

    Point operator+(const Point &p) const {
        return Point(x + p.x, y + p.y, z + p.z);
    }

    Point operator-(const Point &p) const {
        return Point(x - p.x, y - p.y, z - p.z);
    }

    Point operator*(long double k) const {
        return Point(x * k, y * k, z * k);
    }

    Point operator/(long double k) const {
        return Point(x / k, y / k, z / k);
    }

    bool operator<(const Point &p) const {
        return x < p.x || (x == p.x && (y < p.y || (y == p.y && z < p.z)));
    }

    bool operator==(const Point &p) const {
        return fabs(x - p.x) < EPS && fabs(y - p.y) < EPS && fabs(z - p.z) < EPS;
    }
};

long double sp(const Point &a, const Point &b) {
    return a.x * b.x + a.y * b.y + a.z * b.z;
}

Point cp(const Point &a, const Point &b) {
    return Point(a.y * b.z - a.z * b.y, a.z * b.x - a.x * b.z, a.x * b.y - a.y * b.x);
}

long double len(const Point &p) {
    return sqrt(sp(p, p));
}

const int MAXN = 105;
int m, h;
Point p[MAXN], p2d[MAXN * MAXN], ch[MAXN * MAXN];

void add(const Point &a, const Point &b, long double z) {
    if((a.z - z) * (b.z - z) > EPS)
        return;
    if(fabs(a.z - z) < EPS) {
        p2d[m++] = a;
        if(fabs(b.z - z) < EPS)
            p2d[m++] = b;
        return;
    }
    if(fabs(b.z - z) < EPS) {
        p2d[m++] = b;
        return;
    }
    long double k = (z - a.z) / (b.z - a.z);
    p2d[m++] = a + (b - a) * k;
}

void buildCH() {
    sort(p2d, p2d + m);
    m = unique(p2d, p2d + m) - p2d;
    if(m <= 2) {
        h = m;
        for(int i = 0; i < m; i++)
            ch[i] = p2d[i];
        ch[h] = ch[0];
        return;
    }
    int l = 0;
    for(int i = 1; i < m; i++)
        if(p2d[i] < p2d[l])
            l = i;
    swap(p2d[l], p2d[m - 1]);
    h = 1;
    while(h < m) {
        l = 0;
        for(int i = 1; i < m - h; i++) {
            long double cpp = cp(p2d[i] - p2d[m - h], p2d[l] - p2d[m - h]).z;
            if(cpp > EPS || (fabs(cpp) < EPS && len(p2d[i] - p2d[m - h]) < len(p2d[l] - p2d[m - h])))
                l = i;
        }
        if(h > 1 && cp(p2d[m - 1] - p2d[m - h], p2d[l] - p2d[m - h]).z > EPS)
            break;
        h++;
        swap(p2d[l], p2d[m - h]);
    }
    for(int i = 0; i < h; i++)
        ch[i] = p2d[m - i - 1];
    ch[h] = ch[0];
}

long double getS() {
    long double res = 0;
    for(int i = 0; i < h; i++)
        res += cp(ch[i], ch[i + 1]).z;
    return fabs(res) / 2;
}

int main(){
    ios_base::sync_with_stdio(false);
    cout.setf(ios::fixed);
    cout.precision(10);
    int n, zmin, zmax;
    cin >> n >> zmin >> zmax;
    for(int i = 0; i < n; i++)
        cin >> p[i].x >> p[i].y >> p[i].z;
    for(int i = zmin; i <= zmax; i++) {
        m = 0;
        h = 0;
        for(int j = 0; j < n; j++)
            for(int k = j + 1; k < n; k++)
                add(p[j], p[k], i);
        buildCH();
        cout << getS() << '\n';
    }
    return 0;
}
