#include <iostream>
#include <cstdio>
#include <vector>
#include <map>
#include <algorithm>
#include <set>
#include <cstdlib>
#include <cstring>
#include <cmath>

using namespace std;

const long double EPS = 1e-9;

struct Point {
    long double x, y, z;

    Point() {}

    Point(long double x, long double y, long double z): x(x), y(y), z(z) {}

    Point operator+(const Point &p) const {
        return Point(x + p.x, y + p.y, z + p.z);
    }

    Point operator-(const Point &p) const {
        return Point(x - p.x, y - p.y, z - p.z);
    }

    Point operator*(long double k) const {
        return Point(x * k, y * k, z * k);
    }

    Point operator/(long double k) const {
        return Point(x / k, y / k, z / k);
    }
};

long double sp(const Point &a, const Point &b) {
    return a.x * b.x + a.y * b.y + a.z * b.z;
}

Point cp(const Point &a, const Point &b) {
    return Point(a.y * b.z - a.z * b.y, a.z * b.x - a.x * b.z, a.x * b.y - a.y * b.x);
}

long double len(const Point &p) {
    return sqrt(sp(p, p));
}

bool inSemispace(const Point &p, const Point &a, const Point &b, const Point &c) {
    return sp(cp(p - a, b - a), c - a) > EPS;
}

long double getS(const Point &a, const Point &b, const Point &c) {
    return len(cp(b - a, c - a)) / 2;
}

long double getH(const Point &a, const Point &b, const Point &c, const Point &d) {
    return fabs(sp(cp(b - a, c - a), d - a)) / getS(b, c, d) / 2;
}

long double getV(const Point &a, const Point &b, const Point &c, const Point &d) {
    return fabs(sp(cp(b - a, c - a), d - a)) / 6;
}

Point trCenter(const Point &a, const Point &b, const Point &c) {
    return (a + b + c) / 3;
}

Point center(const Point &a, const Point &b, const Point &c, const Point &d) {
    return a + (trCenter(b, c, d) - a) * 0.75;
}

const int MAXN = 105, INF = 1000 * 1000 * 1000 + 5;
Point a[MAXN], b[MAXN];

int main(){
    ios_base::sync_with_stdio(false);
    cout.setf(ios::fixed);
    cout.precision(10);
    int n;
    cin >> n;
    for(int i = 0; i < n; i++) {
        cin >> a[i].x >> a[i].y >> a[i].z;
    }
    Point ac(0, 0, 0);
    long double av = 0;
    for(int i = 0; i < n; i++)
        for(int j = i + 1; j < n; j++)
            for(int k = j + 1; k < n; k++)
                for(int l = k + 1; l < n; l++) {
                    long double v = getV(a[i], a[j], a[k], a[l]);
                    ac = ac + center(a[i], a[j], a[k], a[l]) * v;
                    av += v;
                }
    ac = ac / av;
    long double ad = INF;
    for(int i = 0; i < n; i++)
        for(int j = i + 1; j < n; j++)
            for(int k = j + 1; k < n; k++) {
                bool was[2] = {false, false};
                for(int l = 0; l < n; l++) {
                    if(l == i || l == j || l == k)
                        continue;
                    was[inSemispace(a[l], a[i], a[j], a[k])] = true;
                }
                if(!(was[0] ^ was[1]))
                    continue;
                ad = min(ad, getH(ac, a[i], a[j], a[k]));
            }
    int m;
    cin >> m;
    for(int i = 0; i < m; i++)
        cin >> b[i].x >> b[i].y >> b[i].z;
    Point bc(0, 0, 0);
    long double bv = 0;
    for(int i = 0; i < m; i++)
        for(int j = i + 1; j < m; j++)
            for(int k = j + 1; k < m; k++)
                for(int l = k + 1; l < m; l++) {
                    long double v = getV(b[i], b[j], b[k], b[l]);
                    bc = bc + center(b[i], b[j], b[k], b[l]) * v;
                    bv += v;
                }
    bc = bc / bv;
    long double bd = INF;
    for(int i = 0; i < m; i++)
        for(int j = i + 1; j < m; j++)
            for(int k = j + 1; k < m; k++) {
                bool was[2] = {false, false};
                for(int l = 0; l < m; l++) {
                    if(l == i || l == j || l == k)
                        continue;
                    was[inSemispace(b[l], b[i], b[j], b[k])] = true;
                }
                if(!(was[0] ^ was[1]))
                    continue;
                bd = min(bd, getH(bc, b[i], b[j], b[k]));
            }
    cout << ad + bd << '\n';
    return 0;
}
