#include <iostream>
#include <cstdio>
#include <vector>
#include <map>
#include <algorithm>
#include <set>
#include <cstdlib>
#include <cstring>
#include <cmath>

using namespace std;

const long double EPS = 1e-9;

struct Point {
    long double x, y;

    Point() {}

    Point(long double x, long double y): x(x), y(y) {}

    Point operator+(const Point &p) const {
        return Point(x + p.x, y + p.y);
    }

    Point operator-(const Point &p) const {
        return Point(x - p.x, y - p.y);
    }

    Point operator/(long double k) const {
        return Point(x / k, y / k);
    }
};

long double cp(const Point &a, const Point &b) {
    return a.x * b.y - a.y * b.x;
}
long double sp(const Point &a, const Point &b) {
    return a.x * b.x + a.y * b.y;
}

long double len(const Point &p) {
    return sqrt(sp(p, p));
}

bool onSeg(const Point &p, const Point &a, const Point &b) {
    return fabs(len(p - a) + len(p - b) - len(a - b)) < EPS;
}

long double getS(const Point &a, const Point &b, const Point &c) {
    return fabs(cp(b - a, c - a)) / 2;
}

bool inAngle(const Point &p, const Point &a, const Point &b, const Point &c) {
    return cp(a - b, p - b) * cp(p - b, c - b) > EPS;
}

bool inTriangle(const Point &p, const Point &a, const Point &b, const Point &c) {
    return inAngle(p, a, b, c) && inAngle(p, b, c, a) && inAngle(p, c, a, b);
}

bool checkbb(long double l0, long double r0, long double l1, long double r1) {
    if(l0 > r0)
        swap(l0, r0);
    if(l1 > r1)
        swap(l1, r1);
    return max(l0, l1) < min(r0, r1) + EPS;
}

bool intersect(const Point &a0, const Point &b0, const Point &a1, const Point &b1) {
    long double cpp0 = cp(a1 - a0, b0 - a0) * cp(b0 - a0, b1 - a0),
                cpp1 = cp(a0 - a1, b1 - a1) * cp(b1 - a1, b0 - a1);
    return (cpp0 > EPS && cpp1 > EPS) || (fabs(cpp0) < EPS && cpp1 > EPS) ||
            (fabs(cpp1) < EPS && cpp0 > EPS) || (fabs(cpp0) < EPS && fabs(cpp1) < EPS &&
                                                 checkbb(a0.x, b0.x, a1.x, b1.x) &&
                                                 checkbb(a0.y, b0.y, a1.y, b1.y));
}

bool ok(const Point &p, const Point &a, const Point &b, const Point &c) {
    Point q(0, 0);
    return (int)intersect(p, q, a, b) + (int)intersect(p, q, b, c) + (int)intersect(p, q, c, a) -
            (int)onSeg(a, p, q) - (int)onSeg(b, p, q) - (int)onSeg(c, p, q) == 2;
}

int main(){
    ios_base::sync_with_stdio(false);
    cout.setf(ios::fixed);
    cout.precision(10);
    Point a(0, 0), b(10, 0), c(10, 10), f;
    cin >> f.x >> f.y;
    long double ans = 0;
    for(int i = 0; i < 50; i++) {
        Point d = (a + c) / 2;
        if(ok(f, a, b, d))
            ans += getS(a, b, d);
        if(ok(f, b, c, d))
            ans += getS(b, c, d);
        if(onSeg(f, b, d))
            break;
        if(inTriangle(f, b, a, d))
            c = b;
        else
            a = b;
        b = d;
    }
    cout << ans << '\n';
    return 0;
}
