#include <iostream>
#include <algorithm>
#include <set>
#include <map>
#include <string>
#include <cstdio>
#include <ctime>
#include <cassert>
#include <vector>
#include <queue>
#include <cmath>

using namespace std;

const double EPS = 1e-7;

struct Point {
	double x, y, z;
	Point() {}
	Point(double x, double y, double z) : x(x), y(y), z(z) {}
	void scan() {
		scanf("%lf%lf%lf", &x, &y, &z);
	}
	void scan(double z) {
		(*this).z = z;
		scanf("%lf%lf", &x, &y);
	}
	void print() const {
		printf("%.10lf %.10lf %.10lf\n", x, y, z);
	}
	Point operator -(const Point &A) const {
		return Point(x - A.x, y - A.y, z - A.z);
	}
	double operator %(const Point &A) const {
		return x * A.x + y * A.y + z * A.z;
	}
	Point operator * (const Point &A) const {
		return Point(y * A.z - z * A.y,
			     -x * A.z + z * A.x,
			     x * A.y - y * A.x);
	}
	double len() {
		double d = x * x + y * y + z * z;
		if (fabs(d) < EPS) d = 0;
		return sqrt(d);
	}
	Point normalize(double k = 1) {
		return Point(x * k / len(), y * k / len(), z * k / len());
	}
	double distTo(const Point & A, const Point & B, const Point & C) const {
		Point E = *this;
		Point n = (C - A) * (B - A);
		n = n.normalize();
		return fabs(n % (A - E));
	}
};

int main() {
	freopen("input.txt", "r", stdin);
	double m, h, H;
	Point A, B, C, D, E;
	scanf("%lf%lf%lf", &m, &h, &H);
	E.scan(h);
	A.scan(0);
	B.scan(0);
	C.scan(0);
	D.scan(H);
	double e = min(E.distTo(A, B, C), E.distTo(A, B, D));
	e = min(e, E.distTo(B, C, D));
	e = min(e, E.distTo(A, C, D));
	puts(m - h - e > -EPS ? "YES" : "NO");
}
