#include <iostream>
#include <cstdio>
#include <cassert>
#include <vector>
#include <algorithm>
#include <memory.h>
#include <math.h>

using namespace std;

const int maxn = 128;
const int maxm = 16;
const double eps = 1e-9;

struct pt {
	double x, y;
	
	pt() {}
	pt(double x, double y) : x(x), y(y) {}

	pt operator+ (const pt &p) {
		return pt(x + p.x, y + p.y);
	}

	pt operator- (const pt &p) {
		return pt(x - p.x, y - p.y);
	}

	double operator* (const pt &p) {
		return x * p.y - p.x * y;
	}

	double operator% (const pt &p) {
		return x * p.x + y * p.y;
	}

	pt operator* (double d) {
		return pt(x * d, y * d);
	}

	double d() {
		return sqrt(x * x + y * y);
	}	
};

pt A, B, C, S, T;

bool in(pt A, pt B, pt C, pt O) {
	bool f1 = ((A - O) * (B - O)) > -eps &&
			  ((B - O) * (C - O)) > -eps &&
			  ((C - O) * (A - O)) > -eps;
	
	bool f2 = ((A - O) * (B - O)) < eps &&
			  ((B - O) * (C - O)) < eps &&
			  ((C - O) * (A - O)) < eps;
	return f1 || f2;
}

bool intersect(pt A, pt B, pt C, pt D) {
	pt v = (B - A);
	double s1 = (C - A) * (D - A);
	double s2 = (D - B) * (C - B);
	double s = s1 + s2;
	assert(fabs(s) > eps);
	double k = s1 / s;
	pt E = A + v * k;
	bool res = 1;
	res &= fabs((D - E).d() + (C - E).d() - (C - D).d()) < eps;
	res &= k > -eps && k < 1 - eps;
	return res;
}

bool inter2(pt A, pt B, pt C, pt S, pt T) {
	int cnt = 0;
	cnt += intersect(A, B, S, T);
	cnt += intersect(B, C, S, T);
	cnt += intersect(C, A, S, T);
	return cnt == 2;
}

double solve(pt A, pt B, pt C) {
	pt D = (A + C) * 0.5;
	double s = fabs(((B - A) * (C - A)) * 0.5);
	bool fA = inter2(A, B, D, S, T);
	bool fC = inter2(C, B, D, S, T);
	bool inA = in(A, B, D, T);
	bool inC = in(C, B, D, T);

	if (inA && inC) {
		return s * 0.5;
	}
	double res = 0;
	if (inA) {
		if (fC) res += s * 0.5;
		res += solve(A, D, B);
	} else {
		if (fA) res += s * 0.5;
		res += solve(C, D, B);
	}
	return res;
}

int main() {
	freopen("input.txt", "r", stdin);

	A = pt(0, 0);
	B = pt(10, 0);
	C = pt(10, 10);
	S = pt(0, 0);
	cin >> T.x >> T.y;
	printf("%.10lf\n", solve(A, B, C));

	return 0;
}