#if 1
#include <functional>
#include <algorithm>
#include <iostream>
#include <iterator>
#include <iomanip>
#include <sstream>
#include <numeric>
#include <string>
#include <vector>
#include <bitset>
#include <queue>
#include <stack>
#include <map>
#include <set>
#include <cstdlib>
#include <cassert>
#include <cstring>
#include <cstdio>
#include <cmath>
#include <ctime>
#include <list>
 
using namespace std;
 
typedef long long LL;
typedef long double LD;
typedef pair<int, int> pii;
 
const LD eps = 1e-9;
const LD pi = acos(-1.0);
const LD inf = 1e+9;
 
#define mp make_pair
#define pb push_back
#define X first
#define Y second
 
#define dbg(x) { cerr << #x << " = " << x << endl; }
 
// extended template
#pragma comment(linker, "/STACK:36777216")
typedef vector<int> vi;
typedef vector<vi> vvi;
 
#define forn(i, n) for (int i = 0; i < n; ++i)
#define all(a) (a).begin(), (a).end()
#define rall(a) (a).rbegin(), (a).rend()
 
template<typename T> istream & operator >> (istream &, vector<T> &);
template<typename T> ostream & operator << (ostream &, const vector<T> &);
 
#define START clock_t _clock = clock();
#define END cerr << endl << "time: " << (clock() - _clock) / LD(CLOCKS_PER_SEC) << endl;
 
#define NAME "problem"


LD det(LD a, LD b,
           LD c, LD d)
{
    return a * d - b * c;
}

struct vec3d
{
    LD x, y, z;
    vec3d() {}
    vec3d(LD x, LD y, LD z) : x(x), y(y), z(z) {}
    friend vec3d operator - (const vec3d &a, const vec3d &b)
    {
        return vec3d(a.x - b.x, a.y - b.y, a.z - b.z);
    }
    friend vec3d operator * (const vec3d &a, LD val)
    {
        return vec3d(a.x * val, a.y * val, a.z * val);
    }
    friend vec3d operator + (const vec3d &a, const vec3d &b)
    {
        return vec3d(a.x + b.x, a.y + b.y, a.z + b.z);
    }
    friend LD cross(const vec3d &a, const vec3d &b, const vec3d &c)
    {
        return + a.x * b.y * c.z
               + b.x * c.y * a.z
               + c.x * a.y * b.z
               - c.x * b.y * a.z
               - b.x * a.y * c.z
               - a.x * c.y * b.z;
    }
    
    //friend LD cross(const vec3d &a, const vec3d &b)
    //{
    //    LD d1 = det(a.x, a.y,
    //                b.x, b.y);
    //    LD d2 = det(a.x, a.z,
    //                b.x, b.z);
    //    LD d3 = det(a.y, a.z,
    //                b.y, b.z);
    //    return sqrt(d1 * d1 + d2 * d2 + d3 * d3);
    //}
};

struct vec_t
{
    LD x, y;
    vec_t(LD x, LD y) : x(x), y(y) {}
    vec_t() {}
    friend vec_t operator - (const vec_t &a, const vec_t &b)
    {
        return vec_t(a.x - b.x, a.y - b.y);
    }
    friend LD cross(const vec_t &a, const vec_t &b)
    {
        return a.x * b.y - b.x * a.y;
    }
    friend bool operator < (const vec_t &a, const vec_t &b)
    {
        return mp(a.x, a.y) < mp(b.x, b.y);
    }
    friend bool operator == (const vec_t &a, const vec_t &b)
    {
        return fabs(a.x - b.x) < eps && fabs(a.y - b.y) < eps;
    }
};

int n, zmin, zmax;

vec_t intersect(const vec3d &a, const vec3d &b, LD z)
{
    vec3d u(0, 0, z);
    vec3d v(0, 100, z);
    vec3d w(100, 0, z);

    LD v1 = fabs(cross(u - a, v - a, w - a));
    LD v2 = fabs(cross(v - b, u - b, w - b));

    LD k = v1 / (v1 + v2);

    vec3d c = a + (b - a) * k;
    return vec_t(c.x, c.y);
}

vector<vec_t> convex_hull(vector<vec_t> a)
{
    sort(all(a));
    a.erase(unique(all(a)), a.end());

    int n = a.size();
    vector <vec_t> h;
    for (int i = 0; i < n; i++)
    {
        while (h.size() > 1 && cross(h[h.size() - 1] - h[h.size() - 2], a[i] - h[h.size() - 2]) >= 0 )
            h.pop_back();
        h.pb(a[i]);
    }
    int sz = h.size();
    for (int i = n - 2; i >= 0; i--)
    {
        while (h.size() > sz && cross(h[h.size() - 1] - h[h.size() - 2], a[i] - h[h.size() - 2]) >= 0 )
            h.pop_back();
        h.pb(a[i]);
    }
    if (h.size() > 1)
        h.pop_back();
    return h;
}

LD area(const vector<vec_t> &a)
{
    if (a.size() < 3)
        return 0;
    LD res = 0;
    for (int i = 2; i < a.size(); ++i)
        res += cross(a[i] - a[0], a[i - 1] - a[0]);
    return fabs(res) / 2;
}

void solve()
{
    cin >> n >> zmin >> zmax;
    vector<vec3d> pts(n);
    forn(i, n)
        cin >> pts[i].x >> pts[i].y >> pts[i].z;

    cout.setf(ios::fixed);
    cout.precision(9);
    for (int z = zmin; z <= zmax; ++z)
    {
        vector<vec_t> cur;
        forn(i, n)
            forn(j, n)
            {
                const vec3d &a = pts[i];
                const vec3d &b = pts[j];
                if (fabs(a.z - z) < eps && fabs(b.z - z) < eps)
                {
                    cur.push_back(vec_t(a.x, a.y));
                    cur.push_back(vec_t(b.x, b.y));
                }
                else if (a.z <= z && b.z >= z)
                    cur.push_back(intersect(a, b, z));
            }

        vector<vec_t> res = convex_hull(cur);
        LD ans = area(res);

        cout << ans << endl;
    }
}
 
int main()
{
    // freopen(NAME ".in", "r", stdin); freopen(NAME ".out", "w", stdout);
    // freopen("input.txt", "r", stdin); // freopen("output.txt", "w", stdout);
 
    solve();
 
    return 0;
}
/*******************************************
*******************************************/
 
template<typename T> istream & operator >> (istream &is, vector<T> &v)
{
    forn(i, v.size())
        is >> v[i];
    return is;
}
template<typename T> ostream & operator << (ostream &os, const vector<T> &v)
{
    forn(i, v.size())
        os << v[i] << " ";
    return os;
}
#endif