#if 1
#include <cmath>
#include <iostream>
#include <vector>
#include <cstdio>
using namespace std;

double mysqrt(double a)
{
	if(a < 0) return 0;
	return sqrt(a);
}
bool doubleEqual(double a, double b)
{
	return fabs(a - b) < 1e-9;
}
bool doubleLessOrEqual(double a, double b)
{
	return a < b || doubleEqual(a, b);
}
struct Point
{
	double x, y, z;
	Point() : x(0), y(0), z(0) {}
	Point(double x, double y, double z) : x(x), y(y), z(z) {}
	void scan()
	{
		scanf("%lf%lf%lf", &x, &y, &z);
	}
	void print() const 
	{
		printf("%.10lf %.10lf %.10lf\n", x, y, z);
	}
	double distTo(const Point &p) const
	{
		return (*this - p).length();
	}
	Point operator + (const Point & p) const
	{
		return Point(x + p.x, y + p.y, z + p.z);
	}
	Point operator - (const Point &p ) const
	{
		return Point(x - p.x, y - p.y, z - p.z);
	}
	Point operator - () const
	{
		return Point(-x, -y, -z);
	}
	Point operator * (double k) const 
	{
		return Point(x * k, y * k, z * k);
	}
	Point operator / (double k) const 
	{
		return Point(x / k, y / k, z / k);
	}
	double operator % (const Point & p) const
	{
		return x * p.x + y * p.y + z * p.z;
	}
	Point operator * (const Point &p)
	{
		return Point(	y * p.z - z * p.y,
						z * p.x - x * p.z,
						x * p.y - y * p.x	);
	}
	double length() const
	{
		return mysqrt(*this % *this);
	}
	Point normalize(double k = 1) const
	{
		double len = length();
		return *this * (k / len);
	}
};
void intersectPlane(Point a, Point b, Point c, Point d, Point e, Point &o)
{
	double V1 = (c - a) * (d - a) % (e - a);
	double V2 = (d - b) * (c - b) % (e - b);
	double V = V1 + V2;
	Point v = b - a;
	v = v / V;
	v = v * V1;
	o = a + v;
}

int main()
{
#ifdef _DEBUG
	freopen("input.txt", "r", stdin);
#endif
	double m, h, H; scanf("%lf%lf%lf", &m, &h, &H);
	Point O;
	scanf("%lf%lf", &O.x, &O.z); O.y = h;
	vector<Point> p(4);
	for (int i = 0; i < 4; ++i) scanf("%lf%lf", &p[i].x, &p[i].z);
	p[3].y = H;
	
	vector<Point> n(4);
	n[0] = (p[1] - p[0]) * (p[2] - p[0]);
	n[1] = (p[0] - p[3]) * (p[2] - p[0]);
	n[2] = (p[3] - p[0]) * (p[1] - p[0]);
	n[3] = (p[1] - p[2]) * (p[3] - p[1]);
	for (int i = 0; i < 4; ++i) n[i] = n[i].normalize();

	vector<Point> o(4);
	intersectPlane(O, O - n[0] * (m - h), p[0], p[1], p[2], o[0]);
	intersectPlane(O, O - n[1] * (m - h), p[0], p[3], p[2], o[1]);
	intersectPlane(O, O - n[2] * (m - h), p[0], p[1], p[3], o[2]);
	intersectPlane(O, O - n[3] * (m - h), p[3], p[1], p[2], o[3]);

	bool intersect = false;
	intersect = intersect || doubleLessOrEqual( O.distTo(o[0]), m - h);
	intersect = intersect || doubleLessOrEqual( O.distTo(o[1]), m - h);
	intersect = intersect || doubleLessOrEqual( O.distTo(o[2]), m - h);
	intersect = intersect || doubleLessOrEqual( O.distTo(o[3]), m - h);
	printf("%s\n", intersect ? "YES" : "NO");
	return 0;
}
#endif