#if 1
#include <vector>
#include <iostream>
#include <algorithm>
using namespace std;
struct point 
{
	int x, y, id;
	point() {}
	point(int x, int y, int id) : x(x), y(y), id(id) {}
	friend bool operator < (const point &a, const point &b)
	{
		return make_pair(a.x, a.y) < make_pair(b.x, b.y);
	}
};

int area(point &a, point &b, point &c)
{
	return (a.x - c.x) * (b.y - c.y) - (a.y - c.y) * (b.x - c.x);
}

struct convex_hull
{
	static void make_hull(vector<point> origin, vector<point> &up, vector<point> &down, vector<point> &oth)
	{
		if(origin.size() <= 3)
		{
			up = origin;
			down = vector<point> ();
			oth = vector<point> ();
			return;
		}
		point s = *origin.begin(), e = origin.back();
		up.clear();
		down.clear();
		for (int i = 0; i < origin.size(); ++i)
		{
			int size;
			if(area(e, s, origin[i]) <= 0)
			{
				while((size = up.size()) >= 2 && area(up[size-2], up[size-1], origin[i]) > 0) up.pop_back();
				up.push_back(origin[i]);
			}
			if(area(e, s, origin[i]) >= 0)
			{
				while((size = down.size()) >= 2 && area(down[size-2], down[size-1], origin[i]) < 0) down.pop_back();
				down.push_back(origin[i]);
			}
		}

		int ptr1 = 0, ptr2 = 0;
		oth.clear();
		for (int i = 0; i < origin.size(); ++i)
		{
			bool good = true;
			if(ptr1 < up.size())
			{
				if(origin[i].id == up[ptr1].id)
				{
					ptr1++;
					good = false;
				}
			}
			if(ptr2 < down.size())
			{
				if(origin[i].id == down[ptr2].id)
				{
					ptr2++;
					good = false;
				}
			}
			if(good) oth.push_back(origin[i]);
		}
	}
};

int main()
{
	cin.sync_with_stdio(false);
#ifdef _DEBUG
	freopen("input.txt", "r", stdin);
#endif
	int n; cin >> n;
	vector<point> origin(n);
	for (int i = 0; i < n; ++i)
	{
		cin >> origin[i].x >> origin[i].y;
		origin[i].id = i;
	}
	sort(origin.begin(), origin.end());
	vector<int> ans(n);
	for(int num = 1; ; ++num)
	{
		if(!origin.size()) break;
		vector<point> up, down;
		convex_hull::make_hull(origin, up, down, origin);
		for (int i = 0; i < up.size(); ++i) ans[up[i].id] = num;
		for (int i = 0; i < down.size(); ++i) ans[down[i].id] = num;
	}
	for (int i = 0; i < n; ++i) cout << ans[i] << endl;
}
#endif